/*********************************************************************
 *
 * Relation-preserving mappings program with pre-anaylsis phase
 * to select a suitable element order.
 *
 * Data is via stdin in the following format:
 * Asize
 * Bsize
 * no of relations
 * list of A relations (matrices)
 * list of B relations
 *
 ********************************************************************/

#include "prog.h"


int
main (argc, argv)
     int argc;
     char **argv;
{
  /* Check there is an argument (the parameter file name) */
  if (argc < 2)
    error ("main", "No parameters file given.\n");
  if (argc > 2)
    error ("main", "Only one argument required.\n");
  /* Read parameters file, argv[0] is program name, argv[1] is first arg */
  read_param (argv[1]);
  /* Initialise global data */
  cum_time = 0;
  cum_utime = 0;
  maxtrials = BUDGET;
  maxgoes = MAXGOES;
  max_depth = 0;
  printinterval = PRINTINTERVAL;

  /* Force stdout to be line-buffered: */
  setlinebuf (stdout);
  initrandom ();
  readdata ();
  if (PRINT_DATA) {
    printdata ();
  }
  if (PRE_ANALYSIS) {
    randompi ();
    do_pre_analysis ();
  }
  if (ANALYSE) {
    do_analyse ();
  }
  if (CALCULATE) {
    {
      double result;
      printinfo (1, SA, trials, (double)0);
      (void) printf ("\n--- Now doing full calculation with this perm ---\n");
      setup (SA);
      result = calculate (1, SA, MAXDOUBLE);	/* No budget limit here */
      (void) printf ("maps = %16.0f,\t", result);
      (void) printf ("in %16.0f trials\n", trials);
      printinfo (1, SA, trials, result);
    }
  }				/* fi CALCULATE */
  if (BUSH) {
    do_bush ();
  }
#ifdef lint
  /* keep lint happy about unused functions: */
  {
    int a = 0;
    int b = 0;
    printdata ();
    do_pre_analysis ();
    do_analyse ();
    error ("name", "format %i", a);
    next_pair (&a, &b);
    random_pair (&a, &b);
    (void) find_good_element (a);
  }
#endif
  return (0);
}				/* End of main */
