/*********************************************************************
 *
 * Procedure to read the parameters file and set up the global variables.
 *
 * read-param (filename)
 *
 ********************************************************************/

#include "prog.h"

/* Read a number from the given filehandle. Skip lines which
 * start with #, give an error if EOF is reached
 */
int
read_num (f)
     FILE *f;
{
  char s[2048];
  int c, i;
  int result;
  int ret;

  /* read a line, skipping comment lines.
   * We skip all commas - this allows commas to be part of the input data
   */
  do {
    i = 0;
    /* read a line, up to EOF or nl */
    for (;;) {
      c = getc (f);
      if ((c == EOF) || (c == '\n'))
	break;
      if (c != ',') {
	s[i] = c;
	i++;
      }
    }
    s[i] = '\0';
    line_no++;
  } while ((c != EOF) && (s[0] == '#'));
  if (c == EOF)
    error ("read_num", "Unexpected end of parameter file.\n");
  /* get the integer */
  ret = sscanf (s, "%i", &result);
  if (ret == 0)
    error ("read_num", "Error in format of parameter file, line: %i\n", line_no);
  return (result);
}

void
read_param (filename)
     char *filename;
{
  FILE *f;			/* file handle for parameter file */

  f = fopen (filename, "r");
  if (f == NULL)
    error ("read_param", "Unable to open parameter file `%s'\n", filename);
  line_no = 0;

  /* Read in the parameters */
  PRINT_DATA = read_num (f);
  PRINT_PERM = read_num (f);
  ANALYSE = read_num (f);
  SAMPLES = read_num (f);
  MAXSAMPLES = read_num (f);
  MULT = read_num (f);
  PRE_ANALYSIS = read_num (f);
  MAXGOES = read_num (f);
  BUDGET = read_num (f);
  MAXRETRIES = read_num (f);
  PRINTINTERVAL = read_num (f);
  BETTER = read_num (f);
  TWO_INSERTS = read_num (f);
  CALCULATE = read_num (f);
  BUSH = read_num (f);
  BUSH_BUDGET = read_num (f);
  BUSH_GOES = read_num (f);
  BUSH_RETRIES = read_num (f);
  BUSH_PR_STEP = read_num (f);
  PRINT_BUSH = read_num (f);
  DROP = read_num (f);

  /* Print out the parameters */

  (void) printf ("\nParameter data for this run:\n");
  (void) printf ("ANALYSE = %i", ANALYSE);
  if (ANALYSE) {
    (void) printf (": SAMPLES = %i, MAXSAMPLES = %i, MULT = %i",
		   SAMPLES, MAXSAMPLES, MULT);
  }
  (void) printf ("\n");
  (void) printf ("PRE_ANALYSIS = %i", PRE_ANALYSIS);
  if (PRE_ANALYSIS) {
    (void) printf (": MAXGOES = %i, BUDGET = %i, MAXRETRIES = %i\n",
		   MAXGOES, BUDGET, MAXRETRIES);
    (void) printf ("                  PRINTINTERVAL = %i, BETTER = %i",
		   PRINTINTERVAL, BETTER);
  }
  (void) printf ("\n");
  (void) printf ("CALCULATE = %i\n", CALCULATE);
  (void) printf ("BUSH = %i", BUSH);
  if (BUSH) {
    (void) printf (": BUSH_BUDGET = %i, BUSH_PR_STEP = %i, PRINT_BUSH = %i",
		   BUSH_BUDGET, BUSH_PR_STEP, PRINT_BUSH);
  }
  (void) printf ("\n");
  if (DROP) {
    (void) printf ("Dropping relation: %i\n", DROP);
  }
  (void) printf ("\n");

  /* Close the input file */
  if (fclose (f) != 0)
    error ("read_param", "Unable to close parameter file.\n");

}
