/*********************************************************************
 *
 * Utility functions for relpres
 *
 * error (function_name, format, arg1, arg2...)
 * allocate_3D (x, y, z)
 * do_allocations ()
 * readdata ()
 * setup (n)
 * printdata ()
 * initrandom ()
 * rnd (n)
 * randompi ()
 * insert (x, y)
 * next_pair (a, b)
 * random_pair (a, b)
 *
 ********************************************************************/

#include "prog.h"

/* error should be called like:
 *    error(function_name, format, arg1, arg2...);
 *  Note: function_name and format cannot be declared
 *  separately because of the definition of varargs.
 */
 /*VARARGS0*/
void
error (va_alist)
     va_dcl
{
  va_list args;
  char *fmt;

  va_start (args);
  /* print name of function causing error */
  (void) fprintf (stderr, "ERROR in %s: ", va_arg (args, char *));
  fmt = va_arg (args, char *);
  /* print out remainder of message */
  (void) vfprintf (stderr, fmt, args);
  va_end (args);
  (void) exit (1);
}

short
***
allocate_3D (x, y, z)
     int x, y, z;
{
  int i, j;
  short ***result;
  /* allocate memory for a 3D short array of size 0..x by 0..y by 0..z
   * returns pointer to the array:
   */
  result = (short ***) malloc ((unsigned) ((x + 1) * sizeof (*result)));
  for (i = 0; i <= x; i++) {
    result[i] = (short **) malloc ((unsigned) ((y + 1) * sizeof (**result)));
    for (j = 0; j <= y; j++) {
      result[i][j] = (short *) malloc ((unsigned) ((z + 1) * sizeof (short)));
    }
  }
  return (result);
}

void
do_allocations ()
{
  /* allocate memory for the arrays RA, rel_to_A, rel_A_to
   * RB, psi, pi.
   */

  RA = allocate_3D (R, SA, SA);
  rel_to_A = allocate_3D (R, SA, SA);
  rel_A_to = allocate_3D (R, SA, SA);
  RB = allocate_3D (R, SB, SB);
  tvals = (short *) malloc ((unsigned) ((SB + 1) * sizeof (short)));
  psi = (short *) malloc ((unsigned) ((SA + 1) * sizeof (int)));
  pi = (int *) malloc ((unsigned) ((SA + 1) * sizeof (int)));
  next_bush = (double *) malloc ((unsigned) ((SA + 1) * sizeof (double)));
}

void
readdata ()
{				/* read data from stdin */
  int temp;
  int a, ap, b, bp, rho;
  short **temprel;

  /* Read in the no of relations and sizes of A and B */
  (void) scanf (" %i %i %i", &R, &SA, &SB);

  if (DROP > R)
    error ("readdata", "DROP = %i is greater than the number of relations (%i)\n", DROP, R);

  do_allocations ();

  /* Read in the relations: */
  for (rho = 0; rho < R; rho++) {
    for (a = 1; a <= SA; a++) {
      for (ap = 1; ap <= SA; ap++) {
	(void) scanf (" %i", &temp);
	RA[rho][a][ap] = (short) temp;
      }
    }
  }
  for (rho = 0; rho < R; rho++) {
    for (b = 1; b <= SB; b++) {
      for (bp = 1; bp <= SB; bp++) {
	(void) scanf (" %i", &temp);
	RB[rho][b][bp] = (short) temp;
      }
    }
  }
  /* set up initial pi as identity permutation: */
  for (a = 1; a <= SA; a++) {
    pi[a] = a;
  }
  /* dummy image element to use for psi when mindepth is changed: */
  /* dummy element 0 relates to every other element in every relation */
  for (rho = 0; rho < R; rho++) {
    for (b = 0; b <= SB; b++) {
      RB[rho][b][0] = 1;
      RB[rho][0][b] = 1;
    }
  }
  /* If DROP > 0 then decrement R and swap the dropped relation
   * with the last one
   */
  if (DROP > 0) {
    R--;
    temprel = RA[DROP - 1];
    RA[DROP - 1] = RA[R];
    RA[R] = temprel;
    temprel = RB[DROP - 1];
    RB[DROP - 1] = RB[R];
    RB[R] = temprel;
  }
}				/* End of readdata() */


void
setup (n)
     int n;
{				/* set up relA array with permutation pi: */
  int rho, a, ap, numrel;
  /* set up rel_to_A array with the lists of things related to each element
   * and rel_A_to array with the lists of things each element relates to
   * relA[rho][a][0] is the no. of elements related to a in relation rho,
   * relA[rho][a][1] etc. are the elements themselves.
   * Apply pi to RA for calculation */
  for (rho = 0; rho < R; rho++) {
    for (a = 1; a <= n; a++) {
      numrel = 0;
      for (ap = 1; ap <= n; ap++) {
	if (RA[rho][pi[a]][pi[ap]] == 1) {
	  numrel++;
	  rel_to_A[rho][a][numrel] = ap;
	}
      }
      rel_to_A[rho][a][0] = numrel;
    }
  }
  for (rho = 0; rho < R; rho++) {
    for (a = 1; a <= n; a++) {
      numrel = 0;
      for (ap = 1; ap <= n; ap++) {
	if (RA[rho][pi[ap]][pi[a]] == 1) {
	  numrel++;
	  rel_A_to[rho][a][numrel] = ap;
	}
      }
      rel_A_to[rho][a][0] = numrel;
    }
  }
}				/* End of setup(n) */


void
printdata ()
{
  int a, ap, b, bp, rho;
  (void) printf ("No of relations = %d\n", R);
  (void) printf ("Size of A = %d\n", SA);
  (void) printf ("Size of B = %d\n", SB);
  (void) printf ("A relations:\n");
  for (rho = 0; rho < R; rho++) {
    for (a = 1; a <= SA; a++) {
      for (ap = 1; ap <= SA; ap++) {
	(void) printf ("%s", (RA[rho][a][ap] == 1) ? "1" : "0");
      }
      (void) printf ("\n");
    }
    (void) printf ("\n");
  }
  (void) printf ("\n");
  (void) printf ("B relations:\n");
  for (rho = 0; rho < R; rho++) {
    for (b = 1; b <= SB; b++) {
      for (bp = 1; bp <= SB; bp++) {
	(void) printf ("%s", (RB[rho][b][bp] == 1) ? "1" : "0");
      }
      (void) printf ("\n");
    }
    (void) printf ("\n");
  }
}				/* End of printdata() */

void
initrandom ()
{
  int seed;
  seed = time ((long *) NULL);
  (void) printf ("Random seed = %i\n", seed);
  srandom (seed);
}				/* End of initrandom() */

int
rnd (n)
     int n;
/* return a random integer between 0 and (n - 1)
 * return 0 if n == 0
 */
{
  if (n == 0) {
    return (0);
  } else {
    return (((int) random ()) % n);
  }
}

void
randompi ()
{				/* set up a rendom permutation by permuting pi */
  int i, j, temp;
  for (i = 1; i <= SA; i++) {
    j = (random () % SA) + 1;
    temp = pi[j];
    pi[j] = pi[i];
    pi[i] = temp;
  }
}				/* End of randompi() */


void
insert (x, y)
     int x, y;
{				/* move element at position x to position y in pi */
  int temp, i;
/*
  if ((x < 1) || (x > SA) || (y < 1) || (y > SA)) {
    error ("insert", "Bad x,y values: %i, %i\n", x, y);
  }
*/
  if (x <= y) {
    temp = pi[x];
    for (i = x + 1; i <= y; i++) {
      pi[i - 1] = pi[i];
    }
    pi[y] = temp;
  } else {			/* (y < x) */
    temp = pi[x];
    for (i = x - 1; i >= y; i--) {
      pi[i + 1] = pi[i];
    }
    pi[y] = temp;
  }
}


void
printinfo (mindepth, depth, best_trials, best_result)
     int mindepth, depth;
     double best_trials, best_result;
{
  /* print useful info about current pi, CPU time etc. */
  /* calculate incremental and total CPU time used: */
  struct rusage ruse;
  int tot_time, tot_utime;
  int inc_time, inc_utime;
  /* fill in ruse struct: */
  (void) getrusage (RUSAGE_SELF, &ruse);
  tot_time = ruse.ru_utime.tv_sec + ruse.ru_stime.tv_sec;
  tot_utime = ruse.ru_utime.tv_usec + ruse.ru_stime.tv_usec;
  if (tot_utime >= 1000000) {
    tot_utime -= 1000000;
    tot_time += 1;
  }
  inc_time = tot_time - cum_time;
  inc_utime = tot_utime - cum_utime;
  if (inc_utime < 0) {
    inc_utime += 1000000;
    inc_time -= 1;
  }
  cum_time = tot_time;
  cum_utime = tot_utime;
  (void) printf ("DEPTH/m     best_trials     best_result");
  (void) printf ("     CPU time  increment    total_trials\n");
  (void) printf ("%3i", depth);
  (void) printf (" %3i", mindepth);
  (void) printf ("%16.0f%16.0f", best_trials, best_result);
  (void) printf (" %8i.%.3d", tot_time, (tot_utime / 1000));
  (void) printf (" %6i.%.3d", inc_time, (inc_utime / 1000));
  (void) printf ("%16.0f\n", total_trials);
  if (PRINT_PERM)
    print_perm ();
}				/* End of printinfo(mindepth, depth, best_trials, best_result) */

void
print_perm ()
{
  int i;

  for (i = 1; i <= SA; i++) {
    (void) printf ("%4i", pi[i]);
    if (((i % 19) == 0) || (i == SA)) {
      (void) printf ("\n");
    }
  }
}

void
print_map ()
{
  int i;

  for (i = 1; i <= SA; i++) {
    (void) printf ("%3i->%3i", pi[i], psi[i]);
/*    (void) printf ("%3i->%3i", i, psi[i]); */
    if (i < SA)
      (void) printf (",");
    if ((i % 8) == 0) {
      (void) printf ("\n");
    } else {
      (void) printf (" ");
    }
  }
  (void) printf ("\n");
}

void
next_pair (a, b)
     int *a, *b;
{				/* increment (a,b) to next pair to test */
  do {
    (*a)++;
    if (*a > SA) {
      *a = 1;
      (*b)++;
      if (*b > SA) {
	*b = 1;
      }
    }
  } while (*a == *b);
}

void
random_pair (a, b)
     int *a, *b;
{				/* pick a random pair (a,b) */
  do {
    *a = rnd (SA) + 1;
    *b = rnd (SA) + 1;
  } while (*a == *b);
}
